package analyse;
import java.lang.Math;
import mn.*;
import eqmvt.*;

/**
 * Classe servant  tracer des sections de Poincar. 
 * <p> Pour l'instant, classe en version BETA car la section est forcment  q2 = 0, et les quations du mouvement sont celles du problme  3 corps restreint.
 * @author Mathieu HAVEL, Pierre INIZAN
 * @version 27/01/2007
 * @since Projet Trois Corps 2007
 **/

public class SectionPoincare {
	
	double H0;				// Valeur initiale de l'nergie :
							// dfinit la varit d'nergie sur
							// laquelle on se place. 
	double jacobiH0;
	double pasSectionp10;			// pas d'intgration de p1Initial
	double pasSectionq10;			// pas d'intgration de q1Initial
	double sectionp10min;			// p1Initial minimum
	double sectionp10max;			// p1Initial maximum
	double sectionq10min;			// q1Initial minimum
	double sectionq10max;			// q1Initial maximum
	
	// ===================================================================
	//                           CONSTRUCTEUR
	// ===================================================================
	/**
	   * @param jacobi0 Varit d'nergie sur laquelle on se place (= valeur de l'intgrale de Jacobi)
	   * @param pasSectp10 Pas d'incrmenation de p1Initial (quand on change de trajectoire de Poincar,  J fix)
	   * @param pasSectq10 Pas d'incrmenation de q1Initial (quand on change de trajectoire de Poincar,  J fix)
	   * @param p10SectionMin Valeur minimum de p1Initial
	   * @param p10SectionMax Valeur maximum de p1Initial
	   * @param q10SectionMin Valeur minimum de q1Initial
	   * @param q10SectionMax Valeur maximum de q1Initial
	*/
	public SectionPoincare(double jacobi0, double pasSectp10, double pasSectq10, double p10SectionMin, double p10SectionMax, double q10SectionMin, double q10SectionMax){
		H0 = -0.5 * jacobi0;
		jacobiH0 = jacobi0;
		pasSectionp10 = pasSectp10;
		pasSectionq10 = pasSectq10;
		sectionp10min = p10SectionMin;
		sectionp10max = p10SectionMax;
		sectionq10min = q10SectionMin;
		sectionq10max = q10SectionMax;
	}
	
	// ===================================================================
	//                       PARAMETRES DE LA SECTION
	// ===================================================================
	/**
	   * @return Les paramtres de la section de Poincar (J, q10 min/max, p10 min/max...)
	   * @see SectionPoincare
	*/
	public double[] paramSection(){
		double[] tableauParamSection = new double[6];
		
		tableauParamSection[0] = pasSectionp10;	// pas d'incrmentation de p1Initial
		tableauParamSection[1] = pasSectionq10;	// pas d'incrmentation de q1Initial
		tableauParamSection[2] = sectionp10min;	// p1Initial minimum
		tableauParamSection[3] = sectionp10max;	// p1Initial maximum
		tableauParamSection[4] = sectionq10min;	// q1Initial minimum
		tableauParamSection[5] = sectionq10max;	// q1Initial maximum
		
		return tableauParamSection;
	}
	
	// ===================================================================
	//                      CALCUL DE P2_0 tq J=cste
	// ===================================================================
	/**
	   * Pour l'instant, calcule p20 en utilisant les quations du mouvement du Problme  3 corps restreint.
	   * @param q10 q1Initial de la trajectoire considre
	   * @param q20 q2Initial de la trajectoire considre
	   * @param p10 p1Initial de la trajectoire considre
	   * @param mu Paramtre de masse du problme
	   * @return Les solutions de p2Initial (p20a, p20b) pour assurer J = cste (tant donns p1Initial, q1Initial de la trajectoire)
	   * <p> Retourne donc un tableau [p20a, p20b, solution], o solution = 0. si il n'y a pas de solution, et 1. sinon.
	*/
	public double[] calculp2H0(double q10, double q20, double p10, double mu){
		double[] retourSol = new double[3];
		double solution;	// =0 si pas de solution, =1 sinon.
		
		// on calcule le nouveau p2, pour rester sur la mme varit d'nergie H0
		double nbPa,nbPb,nbPc,nbPd,solPa,solPb;
		double R1=Math.pow(Math.pow(q10+mu,2)+Math.pow(q20,2),0.5);
		double R2=Math.pow(Math.pow(q10+mu-1,2)+Math.pow(q20,2),0.5);
		nbPc = -2*q10;
		nbPa = (1-mu)/R1 + mu/R2;
		nbPb = -2*H0 + Math.pow(p10, 2) + 2*p10*q20 - 2*nbPa;
		nbPd = Math.pow(nbPc,2) - 4*nbPb;
		
		// Test sur les solutions possibles... ou non
		if(nbPd<0){
			solution = 0.;
			solPa = 0.;
			solPb = 0.;
		}
		else if(nbPd==0){
			solution = 1.;
			solPa = -nbPc/2;
			solPb = -nbPc/2;
		}
		else{
			solution = 1.;
			// les deux solutions possibles pour p20
			solPa = (-nbPc + Math.sqrt(nbPd))/2;
			solPb = (-nbPc - Math.sqrt(nbPd))/2;
		}
		
		// ===========================================
		// DEBUGING
		/*
		double jacobiI;
		InterfaceEqMvmt mouvementT;
		mouvementT = UsineEqMvmt.nouveau("EqMvmtPblmTroisCorps");
		System.out.println("H0 = " + H0);
		double[] voyagerR = new double[4];
		double[] mutabB = new double[1];
		voyagerR[0] = q10;
		voyagerR[1] = q20;
		voyagerR[2] = p10;
		voyagerR[3] = solPa;
		mutabB[0] = mu;
		jacobiI = mouvementT.intPrem(voyagerR,mutabB);
		System.out.println("Jacobi P20 = " + jacobiI);
		System.out.println("Solution P20 = " + solution);
		// ===========================================*/
		
		// solutions trouves
		retourSol[0] = solPa;
		retourSol[1] = solPb;
		retourSol[2] = solution;
		
		return retourSol;
	}

	// ===================================================================
	//                   CALCUL DES POINTS DE LA SECTION
	// ===================================================================
	/**
	   * @param voyagerP Tableau contenant les prcdentes solutions de Q1,Q2,P1,P2 pour les 2 valeurs possible de p2Initial : [Q1a, Q2a, P1a, P2a, Q1b, Q2b, P1b, P2b, appel, solution, solutionA, solutionB]
	   * @param mu Paramtre de masse du problme
	   * @param fEqMvmtP Equations du mouvement choisies (pour plus tard...)
	   * @param deltaT Pas d'intgration des quations du mouvement.
	   * @param precision Prcision du calcul.
	   * @param q2Section Section de Poincar choisie (pour l'instant q2Section <=> q2 = O).
	   * @param appelmax Nombre maximum d'itrations avant l'abandon (ie. si la trajectoire ne passe plus par q2Section).
	   * @return Les solutions [Q1,Q2,P1,P2] pour la section  q2Section, et pour les deux valeurs possibles de p2Initial
	   * <p> Retourne donc un tableau [Q1a, Q2a, P1a, P2a, Q1b, Q2b, P1b, P2b, appel, solution, solutionA, solutionB], 
	   * <p> o solution(A|B) = 0. si il n'y a pas de solution, et 1. sinon,
	   * <p> appel est le nombre actualis d'appel  l'intgrateur.
	*/
	public double[] calculVoyager(double[] voyagerP, double mu, String fEqMvmtP, double deltaT, double precision, double q2Section, int appelmax){
		int taille1 = voyagerP.length;
		int taille2 = (voyagerP.length-4)/2;
		int taille3 = voyagerP.length-4;
		int taille4 = voyagerP.length-3;
		double[] voyagerTab=new double[taille1];
		double[] voyagerA = new double[taille2];
		double[] voyagerB = new double[taille2];
		double[] mutab = new double[1];
		double jacobiI;
		InterfaceEqMvmt mouvementT;
		mouvementT = UsineEqMvmt.nouveau(fEqMvmtP);
		double relaxation = 0.9;
		int appel0 = (int) voyagerP[taille3];
		int appelSP = (int) voyagerP[taille3];
		boolean rester = true;
		double solutionA = 0.;	// =0 si pas de solution, =1 sinon.
		double solutionB = 0.;	// =0 si pas de solution, =1 sinon.
		double solution = 0.;	// =0 si pas de solution, =1 sinon.
		
		mutab[0] = mu;
		
		Integrateur monIntegrateurP;    // pour le calcul de la trajectoire 
		monIntegrateurP = new Integrateur(4,mutab,fEqMvmtP);
		
		// Initialisation des q_i, p_i de chaque solution p20
		for(int n=0;n<taille2;n++){
			voyagerA[n] = voyagerP[n];
			voyagerB[n] = voyagerP[n+taille2];
		}

		// Solutions avec p20a
		while(rester){
			// appel de l'intgrateur RK4 symp
			voyagerA = monIntegrateurP.rk4sympf(voyagerA, deltaT, precision, relaxation); //integrateur RK4_SYMP
			appelSP++;
			//rester = false;

			if ((voyagerA[1] < (Math.abs(q2Section)+1.e-2)) && (voyagerA[1] > (-Math.abs(q2Section)-1.e-2))){
				rester = false;
				solutionA = 1.;
			}
			else if((appelSP-appel0)>=appelmax){
				rester = false;
				solutionA = 0.;
			}
		}
		
		// Solutions avec p20b
		rester = true;
		while(rester){
			// appel de l'intgrateur RK4 symp
			voyagerB = monIntegrateurP.rk4sympf(voyagerB, deltaT, precision, relaxation); //integrateur RK4_SYMP
			appelSP++;
			//rester = false;

			if ((voyagerB[1] < (Math.abs(q2Section)+1.e-2)) && (voyagerB[1] > (-Math.abs(q2Section)-1.e-2))){
				rester = false;
				solutionB = 1.;
			}
			else if((appelSP-appel0)>=appelmax){
				rester = false;
				solutionB = 0.;
			}
		}
		
		// Test des solutions (utile, car voyager peut renvoyer 
		// des "mauvaises" solutions d au fait que newton_multidim n'a pas converg)
		jacobiI = mouvementT.intPrem(voyagerA,mutab);
		if((Math.abs(jacobiI)>(1.001*jacobiH0))||(Math.abs(jacobiI)<(0.999*jacobiH0))){
			solutionA = 0.;
		}
		jacobiI = mouvementT.intPrem(voyagerB,mutab);
		if((Math.abs(jacobiI)>(1.001*jacobiH0))||(Math.abs(jacobiI)<(0.999*jacobiH0))){
			solutionB = 0.;
		}
		
		// Solutions totales pour les 2 p20
		for(int k=0;k<taille2;k++){
			voyagerTab[k] = voyagerA[k];
			voyagerTab[k+taille2] = voyagerB[k];
		}
		if(solutionA != solutionB){
			solution = solutionA + solutionB;
		}
		else{
			solution = solutionA;
		}

		voyagerTab[taille3] = appelSP;
		voyagerTab[taille4] = solution;
		voyagerTab[taille1-2] = solutionA;
		voyagerTab[taille1-1] = solutionB;
		
		return voyagerTab;
	}
	
	// ===================================================================
	//                      CHOIX DE LA SECTION
	// ===================================================================
	/**
	   * @return La section de Poincar choisie (pour l'instant toujours q2=0). 
	   * <p> A MODIFIER pour pouvoir tre sur une surface de section de Poincar du problme.
	   * @see SectionPoincare
	*/
	public double choixSection(){
		double q2Section;
		
		q2Section = 0;
		
		return q2Section;
	}
	
}
