package eqmvt;
import java.lang.Math;
/**
 * Classe qui implmente l'quation d'un mouvement, la matrice jacobienne associe, et une intgrale premire du mouvement
 *
 * @author Nicolas DELSATE (version du 17/01/2006) 
 * <p>Mathieu HAVEL, Pierre INIZAN (version du 19/01/2007)
 * @version 19/01/2007
 * @since Projet Trois Corps 2006
 */
public class EqMvmtPblmTroisCorps implements InterfaceEqMvmt{
    
    //////////////////////////////////////////////////////////////////
    //                          eqMvmt                              //
    //////////////////////////////////////////////////////////////////
    
    /**
     * quation du mouvement
     * @param  x            Tableau de la forme [Q1,Q2,P1,P2]
     * @param  parametres   Tableau de paramtres
     * @return   Tableau de la forme [Q1_dot,Q2_dot,P1_dot,P2_dot] 
     * @since Projet Trois Corps 2006
     */
    
    public double[] eqMvmt(double[] x, double[] parametres){
  /*x[0]=Q1
    x[1]=Q2
    x[2]=Q1dot
    x[3]=P2
  */
  
  //tableau (de longueur de x :x.length ) contenant la solution
  double[] xdot=new double[x.length];
  
  //le tableau de parametre n'est qu'un nombre appele mu
  double mu=parametres[0];
  
  //calcul de Qdot1=xdot[0]
  xdot[0]=x[2]+x[1];
  //---Debugging----
  //System.out.println("------------------------");
  //System.out.println("--x[0] = " + x[0] + " et  xdot[0] = " + xdot[0]);
  //---Debugging-
  
  //calcul de Qdot2=xdot[1]
  xdot[1]=x[3]-x[0];
  //---Debugging----
  //System.out.println("--x[1] = " + x[1] + " et  xdot[1] = " + xdot[1]);
  //---Debugging-
  
  //derivee de R1 par rapport a Q1
  double denom1 = Math.pow( Math.pow(x[0]+mu,2.0)+Math.pow(x[1],2.0) , 1.5 );
    
  //derivee de R2 par rapport a Q2
  double denom2 = Math.pow( Math.pow(x[0]+mu-1,2.0)+Math.pow(x[1],2.0) , 1.5 );
    
  //calcul de Pdot1=xdot[2]
  xdot[2] = x[3] - (1.0-mu)*(x[0]+mu) / denom1  - mu*(x[0]+mu-1) / denom2 ;
  //---Debugging----
  //System.out.println("--x[2] = " + x[2] + " et  xdot[2] = " + xdot[2]);
  //---Debugging-
    
  //calcul de Pdot1=xdot[3]
  xdot[3] = -x[2] - (1-mu)*x[1]/denom1  -  mu*x[1]/denom2 ;
    
  //---Debugging----
  //System.out.println("--x[3] = " + x[3] + " et  xdot[3] = " + xdot[3]);
  //---Debugging-
    
  return xdot;
    }
  
  
    //////////////////////////////////////////////////////////////////
    //                          jacMvmt                             //
    //////////////////////////////////////////////////////////////////
  
    /**
     * Matrice jacobienne associe  l'quation du mouvement
     * <p>
     * Utilise indirectement dans l'intgrateur symplectique lors de l'appel  l'algorithme de Newton, qui prend en entre une fonction ET sa matrice jacobienne.
     * @param  x            Tableau de la forme [Q1,Q2,P1,P2]
     * @param  parametres   Tableau de paramtres
     * @return   Tableau de la forme [df1/dx1, df2/dx1,..., dfn/dx1, df1/dx2, ... , dfn/dxn] <P>(ATTENTION : remplissage colonne par colonne)
     * @see mn.Integrateur Integrateur
     * @see fonction.JacNewtSymp JacNewtSymp
     * @since Projet Trois Corps 2007
     */
  
    public double[] jacMvmt(double[] x, double[] parametres){
    
  /*
    x[0]=Q1
    x[1]=Q2
    x[2]=P1
    x[3]=P2
  */
    
  //le tableau de parametre n'est qu'un nombre appele mu
  double mu=parametres[0];
    
  double R1 = Math.pow( Math.pow(x[0]+mu,2.0)+Math.pow(x[1],2.0) , 0.5 );
    
  double R2 = Math.pow( Math.pow(x[0]+mu-1,2.0)+Math.pow(x[1],2.0) , 0.5 );
    
  double[] Df = new double[x.length * x.length];
    
  Df[0] = 0;
  Df[1] = -1;
  Df[2] = -(1-mu)*(Math.pow(R1,2)-3*Math.pow(x[0]+mu,2))/Math.pow(R1,5)-mu*(Math.pow(R2,2)-3*Math.pow(x[0]+mu-1,2))/Math.pow(R2,5); 
  Df[3] = 3*(1-mu)*(x[0]+mu)*x[1]/Math.pow(R1,5) + 3*mu*(x[0]+mu-1)*x[1]/Math.pow(R2,5);
    
  Df[4] = 1;
  Df[5] = 0;
  Df[6] = 3*(1-mu)*(x[0]+mu)*x[1]/Math.pow(R1,5) + 3*mu*(x[0]+mu-1)*x[1]/Math.pow(R2,5); 
  Df[7] = -(1-mu)*(Math.pow(R1,2)-3*Math.pow(x[1],2))/Math.pow(R1,5)-mu*(Math.pow(R2,2)-3*Math.pow(x[1],2))/Math.pow(R2,5);
    
  Df[8] = 1;
  Df[9] = 0;
  Df[10] = 0;
  Df[11] = -1; 
    
  Df[12] = 0;
  Df[13] = 1;
  Df[14] = 1;
  Df[15] = 0;
    
  return Df;
    
    }
  
    //////////////////////////////////////////////////////////////////
    //                          intPrem                             //
    //////////////////////////////////////////////////////////////////
  
    /**
     * Implmente une intgrale premire du mouvement
     * @param  x            Tableau de la forme [X1,X2,...]
     * @param  parametres   Tableau de paramtres
     * @return              Valeur de l'intgrale premire
     * @since Projet Trois Corps 2006
     */
    public double intPrem(double[] x, double[] parametres){
  
  double R1=Math.pow(Math.pow(x[0]+parametres[0],2)+Math.pow(x[1],2),0.5);
  double R2=Math.pow(Math.pow(x[0]+parametres[0]-1,2)+Math.pow(x[1],2),0.5);
  double lagrangien = 0.5* (  Math.pow(x[2]+x[1],2) + Math.pow(x[3]-x[0],2) ) - 0.5 * (Math.pow(x[0],2) + Math.pow(x[1],2) ) - ( (1-parametres[0])/R1 +  parametres[0]/R2 );
    
  double jacobi=-2.0*lagrangien;
    
  return jacobi;
    }

}
